import graphene
from core import ExtendedConnection
from core.schema import OrderedDjangoFilterConnectionField, OpenIMISMutation
from graphene_django import DjangoObjectType
from video.models import Video


class VideoGQLType(DjangoObjectType):
    """
    """

    class Meta:
        model = Video
        exclude_fields = ('row_id',)
        interfaces = (graphene.relay.Node,)
        filter_fields = {
            "video_name": ["exact"],
        }
        connection_class = ExtendedConnection


class Query(graphene.ObjectType):
    videos = OrderedDjangoFilterConnectionField(
        VideoGQLType,
        orderBy=graphene.List(of_type=graphene.String),
    )

class CreateVideoInputType(OpenIMISMutation.Input):
    video_name = graphene.String(required=True)

class CreateVideoMutation(OpenIMISMutation):
    _mutation_module = "claim"
    _mutation_class = "CreateClaimMutation"

    class Input(CreateVideoInputType):
        pass

    @classmethod
    def async_mutate(cls, user, **data):
        Video.objects.create(video_name=data['video_name'])


class Mutation(graphene.ObjectType):
    create_video = CreateVideoMutation.Field()
